/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import java.util.Iterator;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.CollectionUtils.copy;

/**
 * A collection of {@code attribute}s.  Threadsafe and immutable.
 *
 * @see Attribute
 */
public final class Attributes extends UnaryMicrotype<Iterable<Attribute>> implements Iterable<Attribute> {

    private Attributes(final Iterable<Attribute> attributes) {
        super(copy(attributes));
    }

    /**
     * Factory method for making {@code Attributes}.
     *
     * @param attributes the {@code Attribute}s that will populate the collection.
     * @return an {@code Attributes} containing the given {@code Attribute}s
     */
    public static Attributes attributes(Attribute... attributes) {
        return attributes(asList(attributes));
    }

    /**
     * Factory method for making {@code Attributes}.
     *
     * @param attributes the {@code Attribute}s that will populate the collection.
     * @return an {@code Attributes} containing the given {@code Attribute}s
     */
    public static Attributes attributes(final Iterable<Attribute> attributes) {
        return new Attributes(attributes);
    }

    /**
     * Constructs a new {@code Attributes} containing these {@code Attribute}s with the given {@code Attribute} appended.
     *
     * @param attribute the {@code Attribute} to append.
     * @return a new {@code Attributes} containing these {@code Attribute}s with the given {@code Attribute} appended.
     */
    public Attributes append(final Attribute attribute) {
        return new Attributes(CollectionUtils.append(value, attribute));
    }

    @Override
    public Iterator<Attribute> iterator() {
        return value.iterator();
    }
}
