/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import java.net.URI;

/**
 * An XML namespace.
 * <p/>
 * Immutable and threadsafe.
 */
public final class Namespace extends UnaryMicrotype<URI> {

    /**
     * The reserved XML namespace - that which is available by definition in every xml namespace with the local name xml.
     * See http://www.w3.org/TR/REC-xml-names/#xmlReserved for more details.
     */
    public static final Namespace XML = namespace(URI.create("http://www.w3.org/XML/1998/namespace"));

    private Namespace(final URI name) {
        super(name);
    }

    /**
     * Factory method for making a Namespace for the given name.
     *
     * @param name a {@code URI} representing the name of the namespace.
     * @return a {@code Namespace} for the given name.
     */
    public static Namespace namespace(final URI name) {
        return new Namespace(name);
    }

    String asString() {
        return value.toString();
    }
}
