/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;
import java.io.Writer;

/**
 * Formatter for outputting {@code Document}s and {@code Tag}s as human-friendly XML to a {@code Writer}.
 */
public final class PrettyXmlFormatter extends XmlFormatter {

    public PrettyXmlFormatter() {
    }

    @Override
    public void write(final Document document, final Writer writer) throws XmlWriteException {
        StreamResult streamResult = new StreamResult(writer);
        final TransformerHandler transformerHandler;
        try {
            final SAXTransformerFactory saxTransformerFactory = (SAXTransformerFactory) SAXTransformerFactory.newInstance();
            saxTransformerFactory.setAttribute("indent-number", 4);
            transformerHandler = saxTransformerFactory.newTransformerHandler();
        } catch (TransformerConfigurationException e) {
            throw new XmlWriteException("Failed configuring PrettyXmlFormatter", e);
        }
        transformerHandler.getTransformer().setOutputProperty(OutputKeys.INDENT, "yes");
        transformerHandler.setResult(streamResult);
        writeDocument(document, transformerHandler);
    }

}
